﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.DataLoad
{
    public class DataLoad : IDataLoad
    {
        private readonly Tsi1Context _context;
        private readonly ITenantService _tenantService;
        private readonly IUserService _userService;
        private readonly ICourseService _courseService;
        private readonly ISectionService _sectionService;
        private readonly ISectionItemService _sectionItemService;
        private readonly IBedeliaService _bedeliaService;

        public DataLoad(Tsi1Context context,
            ITenantService tenantService,
            IUserService userService,
            ICourseService courseService,
            ISectionService sectionService,
            ISectionItemService sectionItemService,
            IBedeliaService bedeliaService)
        {
            _context = context;
            _tenantService = tenantService;
            _userService = userService;
            _courseService = courseService;
            _sectionService = sectionService;
            _sectionItemService = sectionItemService;
            _bedeliaService = bedeliaService;
        }

        public async Task<ServiceResult<bool>> LoadStudentsInBedelia()
        {
            var users = await (from u in _context.Users
                         join s in _context.Students
                            on u.StudentId equals s.Id
                         select new UserCreateBedeliaDto
                         {
                             IdentityCard = s.IdentityCard,
                             Name = string.Join(" ", u.FirstName, u.LastName),
                             Password = u.Password
                         }).ToListAsync();

            return await _bedeliaService.CreateUsers(users);
        }

        public async Task<ServiceResult<bool>> LoadCoursesInBedelia()
        {
            var courseNames = await _context.Courses.Select(x => x.Name).ToListAsync();

            return await _bedeliaService.CreateCourses(courseNames);
        }

        public async Task LoadDataAsync()
        {
            // TENANTS
            var newTenants = new List<Tenant>
            {
                new Tenant
                {
                    Name = TenantAdmin.Name,
                    Theme = "pulse"
                },
                new Tenant
                {
                    Name = "fing",
                    Theme = "darkly"
                },
                new Tenant
                {
                    Name = "fmed",
                    Theme = "sketchy"

                }
            };
            _context.Tenants.AddRange(newTenants);

            await _context.SaveChangesAsync();

            var tenants = await _context.Tenants.ToListAsync();
            var tenantFing = tenants.FirstOrDefault(x => x.Name == "fing");
            var tenantFmed = tenants.FirstOrDefault(x => x.Name == "fmed");
            var tenantAdminId = tenants.FirstOrDefault(x => x.Name == TenantAdmin.Name).Id;

            // USER TYPES
            var udelarAdminUserType = new UserType
            {
                Name = UserTypes.UdelarAdmin,
            };
            _context.Add(udelarAdminUserType);

            var facultyAdminUserType = new UserType
            {
                Name = UserTypes.FacultyAdmin,
            };
            _context.Add(facultyAdminUserType);

            var professorUserType = new UserType
            {
                Name = UserTypes.Professor,
            };
            _context.Add(professorUserType);

            var studentUserType = new UserType
            {
                Name = UserTypes.Student,
            };
            _context.Add(studentUserType);

            await _context.SaveChangesAsync();

            // USERS
            var users = new List<User>
            {
                new User
                {
                    Id = -1,
                    UserTypeId = udelarAdminUserType.Id,
                    Username = "admin",
                    Password = "admin",
                    FirstName = "admin",
                    LastName = "admin",
                    Email = "admin@mail.com",
                    TenantId = tenantAdminId,
                },

                // FING
                new User
                {
                    Id = -2,
                    UserTypeId = studentUserType.Id,
                    Username = "enzo",
                    Password = "enzo",
                    FirstName = "Enzo",
                    LastName = "Santangelo",
                    Email = "enzo020895@gmail.com",
                    Student = new Student
                    {
                        TenantId = tenantFing.Id,
                        IdentityCard = "4932876-0",
                    },
                    TenantId = tenantFing.Id,
                },

                new User
                {
                    Id = -3,
                    UserTypeId = studentUserType.Id,
                    Username = "lucca",
                    Password = "lucca",
                    FirstName = "Lucca",
                    LastName = "Santangelo",
                    Email = "luccasant95@gmail.com",
                    Student = new Student
                    {
                        TenantId = tenantFing.Id,
                        IdentityCard = "4932877-6",
                    },
                    TenantId = tenantFing.Id,
                },

                new User
                {
                    Id = -4,
                    UserTypeId = studentUserType.Id,
                    Username = "mathias",
                    Password = "mathias",
                    FirstName = "Mathias",
                    LastName = "Martinez",
                    Email = "mathias@mail.com",
                    Student = new Student
                    {
                        IdentityCard = "1234567-8",
                        TenantId = tenantFing.Id,
                    },
                    TenantId = tenantFing.Id,
                },

                new User
                {
                    Id = -5,
                    UserTypeId = professorUserType.Id,
                    Username = "jperez",
                    Password = "jperez",
                    FirstName = "Juan",
                    LastName = "Perez",
                    Email = "jperez@mail.com",
                    Professor = new Professor
                    {
                        Id = -1,
                        TenantId = tenantFing.Id,
                        IdentityCard = "98754342-5",
                    },
                    TenantId = tenantFing.Id,
                },

                new User
                {
                    Id = -6,
                    UserTypeId = professorUserType.Id,
                    Username = "csuarez",
                    Password = "csuarez",
                    FirstName = "Carlos",
                    LastName = "Suarez",
                    Email = "csuarez@mail.com",
                    Professor = new Professor
                    {
                        Id = -2,
                        TenantId = tenantFing.Id,
                        IdentityCard = "2133453-5",
                    },
                    TenantId = tenantFing.Id,
                },

                new User
                {
                    Id = -7,
                    UserTypeId = facultyAdminUserType.Id,
                    Username = "lrodriguez",
                    Password = "lrodriguez",
                    FirstName = "Luis",
                    LastName = "Rodriguez",
                    Email = "lrodriguez@mail.com",
                    TenantId = tenantFing.Id,
                },

                //FMED

                new User
                {
                    Id = -8,
                    UserTypeId = studentUserType.Id,
                    Username = "marcelo",
                    Password = "rodriguez",
                    FirstName = "Marcelo",
                    LastName = "Rodriguez",
                    Email = "marcelo@gmail.com",
                    Student = new Student
                    {
                        TenantId = tenantFmed.Id,
                        IdentityCard = "76347887-0",
                    },
                    TenantId = tenantFmed.Id,
                },

                new User
                {
                    Id = -9,
                    UserTypeId = studentUserType.Id,
                    Username = "rodrigo",
                    Password = "sastre",
                    FirstName = "Rodrigo",
                    LastName = "Sastre",
                    Email = "rsastre@gmail.com",
                    Student = new Student
                    {
                        TenantId = tenantFmed.Id,
                        IdentityCard = "73642873-6",
                    },
                    TenantId = tenantFmed.Id,
                },

                new User
                {
                    Id = -10,
                    UserTypeId = professorUserType.Id,
                    Username = "dgonzalez",
                    Password = "dgonzalez",
                    FirstName = "Diego",
                    LastName = "Gonzalez",
                    Email = "dgonzalez@mail.com",
                    Professor = new Professor
                    {
                        Id = -3,
                        TenantId = tenantFmed.Id,
                        IdentityCard = "897349827-5",
                    },
                    TenantId = tenantFmed.Id,
                },

                new User
                {
                    Id = -11,
                    UserTypeId = professorUserType.Id,
                    Username = "gfernandez",
                    Password = "gfernandez",
                    FirstName = "Guillermo",
                    LastName = "Fernandez",
                    Email = "gfernandez@mail.com",
                    Professor = new Professor
                    {
                        Id = -4,
                        TenantId = tenantFmed.Id,
                        IdentityCard = "782698091-5",
                    },
                    TenantId = tenantFmed.Id,
                },

                new User
                {
                    Id = -12,
                    UserTypeId = facultyAdminUserType.Id,
                    Username = "jsilva",
                    Password = "jsilva",
                    FirstName = "Jorge",
                    LastName = "Silva",
                    Email = "jsilva@mail.com",
                    TenantId = tenantFmed.Id,
                },
            };

            _context.Users.AddRange(users);

            try
            {
                await _context.SaveChangesAsync();
            }
            catch (Exception ex)
            {

                Console.WriteLine(ex.Message);
            }

            // SECTION ITEM TYPES
            var sectionItemTypeFile = new SectionItemType
            {
                Name = SectionItemTypes.File,
            };
            _context.SectionItemTypes.Add(sectionItemTypeFile);

            var sectionItemTypeForum = new SectionItemType
            {
                Name = SectionItemTypes.Forum,
            };
            _context.SectionItemTypes.Add(sectionItemTypeForum);

            var sectionItemTypeSurvey = new SectionItemType
            {
                Name = SectionItemTypes.Survey,
            };
            _context.SectionItemTypes.Add(sectionItemTypeSurvey);

            await _context.SaveChangesAsync();


            // EVALUATION TYPES
            var examEvaluationType = new EvaluationType
            {
                Name = "Exam"
            };
            _context.EvaluationTypes.Add(examEvaluationType);

            var laboratoryEvaluationType = new EvaluationType
            {
                Name = "Laboratory"
            };
            _context.EvaluationTypes.Add(laboratoryEvaluationType);

            var midTermEvaluationType = new EvaluationType
            {
                Name = "MidTerm"
            };
            _context.EvaluationTypes.Add(midTermEvaluationType);

            await _context.SaveChangesAsync();


            // COURSES  
            var courses = new List<Course>
            {
                new Course
                {
                    Id = -1,
                    Name = "Calculo 1",
                    TenantId = tenantFing.Id,
                    IsPublic = true,
                    HasPassword = true,
                    Password = "fing",                   
                    
                },

                new Course
                {
                    Id = -2,
                    Name = "GAL 1",
                    TenantId = tenantFing.Id,
                    IsPublic = true,
                },

                new Course
                {
                    Id = -3,
                    Name = "Fisica 1",
                    TenantId = tenantFing.Id,
                    IsPublic = false,
                },

                new Course
                {
                    Id = -11,
                    Name = "Neurociencias",
                    TenantId = tenantFmed.Id,
                    IsPublic = true,
                },

                new Course
                {
                    Id = -21,
                    Name = "Cardiovascular y Respiratorio",
                    TenantId = tenantFmed.Id,
                    IsPublic = false,
                    HasPassword = true,
                    Password = "fmed"
                },

            };

            _context.Courses.AddRange(courses);

            await _context.SaveChangesAsync();

            // PROFESSOR COURSES

            var professorCourses = new List<ProfessorCourse>
            {
                new ProfessorCourse
                {
                    CourseId = -1,
                    ProfessorId = -1,
                },

                new ProfessorCourse
                {
                    CourseId = -2,
                    ProfessorId = -1,
                },

                new ProfessorCourse
                {
                    CourseId = -3,
                    ProfessorId = -1,
                },

                new ProfessorCourse
                {
                    CourseId = -1,
                    ProfessorId = -2,
                },

                new ProfessorCourse
                {
                    CourseId = -2,
                    ProfessorId = -2,
                },

                new ProfessorCourse
                {
                    CourseId = -11,
                    ProfessorId = -3,
                },

                new ProfessorCourse
                {
                    CourseId = -21,
                    ProfessorId = -3,
                },

                new ProfessorCourse
                {
                    CourseId = -11,
                    ProfessorId = -4,
                },
            };

            _context.ProfessorCourses.AddRange(professorCourses);

            await _context.SaveChangesAsync();

            // SECTIONS
            var section1 = new Section
            {
                CourseId = -1,
                Name = "General",
                Order = 1,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {
                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Novedades"
                        }
                    },
                    new SectionItem
                    {
                        Order = 2,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "General"
                        }
                    },
                }
            };
            _context.Sections.Add(section1);

            var section2 = new Section
            {
                CourseId = -2,
                Name = "General",
                Order = 1,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {

                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Novedades"
                        }
                    }
                }
            };
            _context.Sections.Add(section2);

            var section3 = new Section
            {
                CourseId = -1,
                Name = "Tema 1",
                Order = 2,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {
                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Tema 1"
                        }
                    }
                }
            };
            _context.Sections.Add(section3);

            var section4 = new Section
            {
                CourseId = -3,
                Name = "General",
                Order = 1,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {
                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Novedades"
                        }
                    },
                    new SectionItem
                    {
                        Order = 2,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "General"
                        }
                    },
                }
            };
            _context.Sections.Add(section4);


            var section5 = new Section
            {
                CourseId = -11,
                Name = "General",
                Order = 1,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {
                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Novedades"
                        }
                    },
                    new SectionItem
                    {
                        Order = 2,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "General"
                        }
                    },
                }
            };
            _context.Sections.Add(section5);

            var section6 = new Section
            {
                CourseId = -21,
                Name = "General",
                Order = 1,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {
                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Novedades"
                        }
                    },
                    new SectionItem
                    {
                        Order = 2,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "General"
                        }
                    },
                }
            };
            _context.Sections.Add(section6);

            await _context.SaveChangesAsync();

            // ANSWER OPTIONS
            var answerOptions = new List<AnswerOption>
            {
                new AnswerOption
                {
                    Name = "Excelente"
                },
                new AnswerOption
                {
                    Name = "Conforme"
                },
                new AnswerOption
                {
                    Name = "Neutro"
                },
                new AnswerOption
                {
                    Name = "No Conforme"
                },
                new AnswerOption
                {
                    Name = "Mal"
                },
            };

            _context.AnswerOptions.AddRange(answerOptions);
            await _context.SaveChangesAsync();


            // Actividades
            var activities = new List<Activity>
            {
                new Activity
                {
                    CourseId = -1,
                    Date = new DateTime(2020, 12, 19),
                    IsVideoConference = false,
                    Name = "Primer Parcial",
                    Evaluation = new Evaluation
                    {
                        EvaluationTypeId = midTermEvaluationType.Id,
                    }
                },

                new Activity
                {
                    CourseId = -1,
                    Date = new DateTime(2020, 12, 28),
                    IsVideoConference = false,
                    Name = "Segundo Parcial",
                    Evaluation = new Evaluation
                    {
                        EvaluationTypeId = midTermEvaluationType.Id,
                    }
                },

                new Activity
                {
                    CourseId = -2,
                    Date = new DateTime(2020, 12, 29),
                    IsVideoConference = false,
                    Name = "Examen",
                    Evaluation = new Evaluation
                    {
                        EvaluationTypeId = examEvaluationType.Id,
                    }
                },

                new Activity
                {
                    CourseId = -3,
                    Date = new DateTime(2020, 12, 27),
                    IsVideoConference = false,
                    Name = "Examen",
                    Evaluation = new Evaluation
                    {
                        EvaluationTypeId = examEvaluationType.Id,
                    }
                },

                new Activity
                {
                    CourseId = -11,
                    Date = new DateTime(2020, 12, 16),
                    IsVideoConference = false,
                    Name = "Primer Parcial",
                    Evaluation = new Evaluation
                    {
                        EvaluationTypeId = midTermEvaluationType.Id,
                    }
                },

                new Activity
                {
                    CourseId = -11,
                    Date = new DateTime(2020, 12, 27),
                    IsVideoConference = false,
                    Name = "Segundo Parcial",
                    Evaluation = new Evaluation
                    {
                        EvaluationTypeId = midTermEvaluationType.Id,
                    }
                },

                new Activity
                {
                    CourseId = -21,
                    Date = new DateTime(2020, 12, 24),
                    IsVideoConference = false,
                    Name = "Examen",
                    Evaluation = new Evaluation
                    {
                        EvaluationTypeId = examEvaluationType.Id,
                    }
                },

            };

            _context.Activities.AddRange(activities);
            await _context.SaveChangesAsync();

            // SURVEYS

            var surveys = new List<Survey>
            {
                new Survey
                {
                    IsGlobal = true,
                    Name = "Encuesta Fing",
                    Tenant = tenantFing,
                    SurveyQuestions = new List<SurveyQuestion>
                    {
                        new SurveyQuestion
                        {
                            Question = "¿Opinión sobre la Facultad de Ingeniería?"
                        },

                        new SurveyQuestion
                        {
                            Question = "¿Opinion sobre las carreras de grado?"
                        },

                        new SurveyQuestion
                        {
                            Question = "¿Opinion sobre las carreras de posgrado?"
                        },
                    }
                },

                new Survey
                {
                    IsGlobal = true,
                    Name = "Encuesta Fmed",
                    Tenant = tenantFmed,
                    SurveyQuestions = new List<SurveyQuestion>
                    {
                        new SurveyQuestion
                        {
                            Question = "¿Opinión sobre la Facultad de Medicina?"
                        },

                        new SurveyQuestion
                        {
                            Question = "¿Opinion sobre las carreras de grado?"
                        },

                        new SurveyQuestion
                        {
                            Question = "¿Opinion sobre las carreras de posgrado?"
                        },
                    }
                },
            };

            _context.Surveys.AddRange(surveys);
            await _context.SaveChangesAsync();

            // Communications

            var calculo1 = await _context.Courses.FirstOrDefaultAsync(x => x.Id == -1);
            var gal1 = await _context.Courses.FirstOrDefaultAsync(x => x.Id == -2);
            var fisica1 = await _context.Courses.FirstOrDefaultAsync(x => x.Id == -3);
            var neuroCiencias = await _context.Courses.FirstOrDefaultAsync(x => x.Id == -11);
            var cardiovascular = await _context.Courses.FirstOrDefaultAsync(x => x.Id == -21);

            var communications = new List<Communication>
            {
                new Communication
                {
                    IsGlobal = true,
                    Tenant = tenantFing,
                    Text = "La facultad permanecera cerrada el día 25/12/2020"
                },

                new Communication
                {
                    IsGlobal = true,
                    Tenant = tenantFmed,
                    Text = "La facultad permanecera cerrada el día 01/01/2021"
                },

                new Communication
                {
                    IsGlobal = false,
                    Course = calculo1,
                    Text = "Se suspende la clase del día 22/12/2020"
                },

                new Communication
                {
                    IsGlobal = false,
                    Course = gal1,
                    Text = "Se suspende la clase del día 17/12/2020"
                },

                new Communication
                {
                    IsGlobal = false,
                    Course = neuroCiencias,
                    Text = "Se suspende la clase del día 26/12/2020"
                },
                new Communication
                {
                    IsGlobal = false,
                    Course = cardiovascular,
                    Text = "Se suspende la clase del día 23/12/2020"
                },
            };

            _context.Communications.AddRange(communications);
            await _context.SaveChangesAsync();
        }
    }
}